@@ -1,16 +1,21 @@
+from sklearn.base import BaseEstimator, TransformerMixin, MultiOutputMixin
+from sklearn.metrics import check_scoring
+
 from bgp import flow
 from bgp.base import SymbolSet
 from bgp.calculation.scores import calculate_y_unpack
 from bgp.calculation.translate import general_expr
 from bgp.flow import MutilMutateLoop
-from sklearn.base import BaseEstimator, TransformerMixin, MultiOutputMixin
-from sklearn.metrics import check_scoring
 
 
 class SymbolLearning(BaseEstimator, MultiOutputMixin, TransformerMixin):
     """One simplify API for flow.\n
     The detailed functions should turn to the loop of bgp.flow.\n
-    The SymbolLearning is time costing and are not suit for GridSearchCV, the cross_validate are builtin """
+    The SymbolLearning is time costing and are not suit for GridSearchCV, the cross_validate are builtin.
+    The classification problems should using "classification"=True, and set the suit classification metrics for "scoring"
+    and "score_pen" carefully.
+    This code does not check and identity the data.
+     """
 
     def __str__(self):
         return str(self.loop)
@@ -126,6 +131,11 @@ class SymbolLearning(BaseEstimator, MultiOutputMixin, TransformerMixin):
                 def func(ind):\n
                     c = ind.fitness.values[0]>=0.90
                     return c
+        details:bool
+            return expr and predi_y or not.
+
+        classification: bool
+            classification or not.
         """
         self.args = args
         self.kwargs = kwargs
@@ -137,7 +147,8 @@ class SymbolLearning(BaseEstimator, MultiOutputMixin, TransformerMixin):
         self.loop = loop
 
     def fit(self, X=None, y=None, c=None, x_group=None, x_dim=1, y_dim=1, c_dim=1, x_prob=None,
-            c_prob=None, pset=None, power_categories=(2, 3, 0.5), categories=("Add", "Mul", "Sub", "Div")):
+            c_prob=None, pset=None, power_categories=(2, 3, 0.5), categories=("Add", "Mul", "Sub", "Div"),
+            warm_start=False, new_gen=None):
         """
 
         If need more self-definition, use one defined SymbolSet object to pset.\n
@@ -188,7 +199,15 @@ class SymbolLearning(BaseEstimator, MultiOutputMixin, TransformerMixin):
 
         pset:SymbolSet
             See Also SymbolSet
-
+        warm_start: bool
+            warm start or not.
+            Note:
+                If you offer pset in advance by user, please check carefully the feature numbers,especially when use "re_Tree.
+                because the new fatures are add.
+            Reference:
+                CalculatePrecisionSet.update_with_X_y
+        new_gen: None,int
+            warm_start generation.
 
         """
         # try to find pest form args,kwargs
@@ -205,17 +224,42 @@ class SymbolLearning(BaseEstimator, MultiOutputMixin, TransformerMixin):
 
         if pset is None:
             # one simple pset are generate with no dimension calculation, But just with x_group.\n
+
             if X is not None and y is not None:
                 pset = SymbolSet()
                 pset.add_features_and_constants(X, y, c, x_dim=x_dim, y_dim=y_dim, c_dim=c_dim, x_prob=x_prob,
                                                 c_prob=c_prob, x_group=x_group, feature_name=None)
                 pset.add_operations(power_categories=power_categories,
                                     categories=categories)
+
+            elif hasattr(self.loop, "gen"):
+                pass
+            else:
+                raise ValueError("The pset should be defined or the X and Y should be offered.")
+        ####################################
+
+        if warm_start:
+            assert hasattr(self.loop, "gen"), "Before the warm_start, Need fit at least one time"
+            if X is not None and y is not None:
+                self.loop.cpset.update_with_X_y(X, y)
+            elif pset:
+                # the warm_start are not compacting with "re_Tree"
+                self.loop.cpset.update(pset)
             else:
                 raise ValueError("The pset should be defined or the X and Y should be offered.")
 
-        self.loop = self.loop(pset, *self.args, **self.kwargs)
-        hall = self.loop.run()
+            self.loop.re_fresh_by_name()
+
+            hall = self.loop.run(warm_start=True, new_gen=new_gen)
+        else:
+            if hasattr(self.loop, "gen"):
+                loops = self.loop.__class__
+                self.loop = loops(pset, *self.args, **self.kwargs)
+            else:
+                self.loop = self.loop(pset, *self.args, **self.kwargs)
+
+            hall = self.loop.run()
+
         self.best_one = hall.items[0]
         try:
             expr = general_expr(self.best_one.coef_expr, self.loop.cpset)
@@ -293,8 +337,9 @@ if __name__ == "__main__":
     y = data["target"]
     c = [6, 3, 4]
 
-    sl = SymbolLearning(loop=None, pop=50, gen=3, cal_dim=True, re_hall=2, add_coef=True, cv=2, random_state=1,store = r"/data/home/wangchangxin"
-                        )
+    sl = SymbolLearning(loop=None, pop=50, gen=3, cal_dim=False, re_hall=2, add_coef=True, cv=1, random_state=2,
+                        re_Tree=1, details=True, store=r"/data/home/wangchangxin")
     sl.fit(x, y, c=c, x_group=[[1, 3], [0, 2], [4, 7]])
-    score = sl.score(x, y, "r2")
-    print(sl.expr)
+    # sl.fit(x, y, c=c, x_group=[[1, 3], [0, 2], [4, 7]], warm_start=True)
+    # score = sl.score(x, y, "r2")
+    # print(sl.expr)
